package data_manage_permission

import (
	"eta/eta_api/utils"
	"fmt"
	"github.com/beego/beego/v2/client/orm"
	"time"
)

// DataPermissionNoAuthRecord
// @Description: 资产数据权限设置记录表
type DataPermissionNoAuthRecord struct {
	DataPermissionNoAuthRecordId int64     `json:"data_permission_no_auth_record_id" orm:"column(data_permission_no_auth_record_id);pk"` // 资产数据操作记录id
	Source                       int32     `json:"source"`                                                                               // 数据来源，1：手工数据指标 2：钢联化工数据库 3：ETA指标库 4：ETA预测指标 5：图库 6：ETA表格
	SubSource                    int32     `json:"sub_source"`                                                                           // 子来源 ：ETA表格中的各种表格类型，以及图表的来源（这个是后续的扩展方向）
	OpUniqueCode                 string    `json:"op_unique_code"`                                                                       // 操作的唯一编码，主要是记录统一操作的日志
	DataId                       string    `json:"data_id"`                                                                              // 资产id（指标、图表、表格）
	DataCode                     string    `json:"data_code"`                                                                            // 资产code（指标、图表、表格）
	DataName                     string    `json:"data_name"`                                                                            // 资产名称（指标、图表、表格）
	SysUserId                    int32     `json:"sys_user_id"`                                                                          // 系统用户id
	CreateTime                   time.Time `json:"create_time"`                                                                          // 创建时间
}

type DataItem struct {
	DataId   string `json:"data_id"`   // 资产id（指标、图表、表格）
	DataCode string `json:"data_code"` // 资产code（指标、图表、表格）
	DataName string `json:"data_name"` // 资产名称（指标、图表、表格）
}

// AddRecordBySourceAndDataIdList
// @Description: 添加记录
// @author: Roc
// @datetime 2024-04-07 15:25:49
// @param source int
// @param subSource int
// @param dataIdList []string
// @param dataList []DataItem
// @param addUserIdList []int
// @param uniqueCode
// @param content string
// @param opUserId int
// @return err error
func AddRecordBySourceAndDataIdList(source, subSource int, dataIdList []string, dataList []DataItem, addUserIdList []int, uniqueCode, content string, opUserId int) (err error) {
	num := len(dataList)
	if num <= 0 {
		return
	}
	userNum := len(addUserIdList)
	if userNum <= 0 {
		return
	}
	o, err := orm.NewOrmUsingDB("data").Begin()
	if err != nil {
		return
	}
	defer func() {
		if err != nil {
			_ = o.Rollback()
		} else {
			_ = o.Commit()
		}
	}()

	// 根据指标获取已经存在的未授权记录
	var existList []*DataPermissionNoAuthRecord
	sql := `SELECT * FROM data_permission_no_auth_record WHERE source = ? AND sub_source = ? AND data_id in (` + utils.GetOrmInReplace(num) + `)`
	_, err = o.Raw(sql, source, subSource, dataIdList).QueryRows(&existList)
	if err != nil {
		return
	}

	// 已经标记了的数据
	existMap := make(map[int32]map[string]*DataPermissionNoAuthRecord)
	for _, v := range existList {
		tmpUserExistMap, ok := existMap[v.SysUserId]
		if !ok {
			tmpUserExistMap = make(map[string]*DataPermissionNoAuthRecord)
		}

		key := fmt.Sprint(v.Source, "_", v.SubSource, "_", v.DataId)
		tmpUserExistMap[key] = v
		existMap[v.SysUserId] = tmpUserExistMap
	}

	addMessageList := make([]*DataPermissionMessage, 0)
	addRecordList := make([]*DataPermissionNoAuthRecord, 0)
	for _, userId := range addUserIdList {
		isAdd := false

		tmpUserExistMap, userExistOk := existMap[int32(userId)]

		for _, dataitem := range dataList {
			// 判断是否已经存在，已经存在就过滤
			if userExistOk {
				key := fmt.Sprint(source, "_", subSource, "_", dataitem.DataId)
				_, ok := tmpUserExistMap[key]
				if ok {
					continue
				}
			}

			isAdd = true
			addRecordList = append(addRecordList, &DataPermissionNoAuthRecord{
				DataPermissionNoAuthRecordId: 0,
				Source:                       int32(source),
				SubSource:                    int32(subSource),
				OpUniqueCode:                 uniqueCode,
				DataId:                       dataitem.DataId,
				DataCode:                     dataitem.DataCode,
				DataName:                     dataitem.DataName,
				SysUserId:                    int32(userId),
				CreateTime:                   time.Now(),
			})
		}

		if isAdd {
			addMessageList = append(addMessageList, &DataPermissionMessage{
				DataPermissionMessageId: 0,
				SendUserId:              int32(opUserId),
				ReceiveUserId:           int32(userId),
				Content:                 content,
				Remark:                  content,
				OpType:                  1,
				Source:                  int32(source),
				SubSource:               int32(subSource),
				OpUniqueCode:            uniqueCode,
				IsRead:                  0,
				CreateTime:              time.Now(),
				ModifyTime:              time.Now(),
			})
		}
	}

	if len(addMessageList) > 0 {
		_, err = o.InsertMulti(500, addMessageList)
		if err != nil {
			return
		}
	}

	if len(addRecordList) > 0 {
		_, err = o.InsertMulti(500, addRecordList)
		if err != nil {
			return
		}
	}

	return
}

// DeleteRecordBySourceAndDataIdList
// @Description: 根据来源和数据id列表删除记录
// @author: Roc
// @datetime 2024-04-07 14:47:37
// @param source int
// @param subSource int
// @param dataIdList []string
// @return err error
func DeleteRecordBySourceAndDataIdList(source, subSource int, dataIdList []string) (err error) {
	num := len(dataIdList)
	if num <= 0 {
		return
	}
	o := orm.NewOrmUsingDB("data")

	sql := `DELETE FROM data_permission_no_auth_record WHERE source = ? AND sub_source = ? AND data_id in (` + utils.GetOrmInReplace(num) + `)`
	_, err = o.Raw(sql, source, subSource, dataIdList).Exec()

	return
}
